/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/. */

var Cc = Components.classes;
var Ci = Components.interfaces;
var SEAMONKEY_ID = "{92650c4d-4b8e-4d2a-b7eb-24ecf4f6b63a}";
var SAP_paneload = null;
const SAP_prefWindowType = "mozilla:preferences";
const SAP_prefName = "services.sync.engine.addons";
const SAP_prefID = "engine.addons";

Components.utils.import("resource://gre/modules/Services.jsm");

function addToSyncPane(aWin) {
  let syncPane = aWin.document.getElementById("sync_pane");
  let syncPrefs = syncPane.getElementsByTagName("preferences")[0];
  let addonsEngine = aWin.document.getElementById("engine.addons");
  if (addonsEngine) { // already there, don't add again
    addonsEngine.removeAttribute("readonly");
  } else {
    addonsEngine = aWin.document.createElement("preference");
    addonsEngine.setAttribute("id", SAP_prefID);
    addonsEngine.setAttribute("name", SAP_prefName);
    addonsEngine.setAttribute("type", "bool");
    syncPrefs.insertBefore(addonsEngine, syncPrefs.firstChild);
  }

  let syncEnginesList = aWin.document.getElementById("syncEnginesList");
  let newEngine = aWin.document.createElement("listitem");
  newEngine.setAttribute("type", "checkbox");
  newEngine.setAttribute("label", "Add-ons");
  newEngine.setAttribute("accesskey", "A");
  newEngine.setAttribute("preference", SAP_prefID);
  newEngine.setAttribute("checked", Services.prefs.getBoolPref(SAP_prefName));
  syncEnginesList.insertBefore(newEngine, syncEnginesList.firstChild);
}

function loadIntoWindow(aWin) {
  if (!aWin)
    return;

  let syncPane = aWin.document.getElementById("sync_pane");
  if (syncPane) {
    if (syncPane.firstChild) { // pane is set up
      addToSyncPane(aWin);
    } else {
      SAP_paneload = function() {
        syncPane.removeEventListener("paneload", SAP_paneload, false);
        addToSyncPane(aWin);
      };
      syncPane.addEventListener("paneload", SAP_paneload, false);
    }
  }
}

function unloadFromWindow(aWin) {
  if (!aWin)
    return;

  let syncPane = aWin.document.getElementById("sync_pane");
  if (syncPane) {
    if (SAP_paneload)
      syncPane.removeEventListener("paneload", SAP_paneload, false);

    if (syncPane.firstChild) { // pane is set up
      let syncEnginesList = aWin.document.getElementById("syncEnginesList");
      syncEnginesList.removeChild(syncEnginesList.firstChild);

      let addonsEngine = aWin.document.getElementById(SAP_prefID);
      if (addonsEngine)
        addonsEngine.setAttribute("readonly", true);
    }
  }
}

function isSM28(aData) {
  var appInfo = Services.appinfo;
  var versionChecker = Cc["@mozilla.org/xpcom/version-comparator;1"]
                        .getService(Ci.nsIVersionComparator);
  if (appInfo.ID == SEAMONKEY_ID &&
      versionChecker.compare(appInfo.version, "2.8a") >= 0 &&
      versionChecker.compare(appInfo.version, "2.9a") < 0)
    return true;

  Components.utils.reportError(aData.id + " requires SeaMonkey 2.8; aborting.");
  return false;
}

var windowListener = {
  onOpenWindow: function(aWin) {
    // Wait for the window to finish loading
    let domWindow = aWin.QueryInterface(Ci.nsIInterfaceRequestor).getInterface(Ci.nsIDOMWindow);
    domWindow.addEventListener("load", function SAP_domWindowLoad() {
      domWindow.removeEventListener("load", SAP_domWindowLoad, false);
      let windowtype = domWindow.document.documentElement.getAttribute("windowtype");
      if (windowtype == SAP_prefWindowType)
        loadIntoWindow(domWindow);
    }, false);
  },
  onCloseWindow: function(aWin) { },
  onWindowTitleChange: function(aWin, aTitle) { }
};

/* bootstrap.js API */

function startup(aData, aReason) {
  if (!isSM28(aData))
    return;

  // Set prefs
  Services.prefs.setCharPref("services.sync.registerEngines",
                             "Bookmarks,Form,History,Password,Prefs,Tab,Addons");
  Services.prefs.setBoolPref("services.sync.prefs.sync.addons.ignoreUserEnabledChanges", true);

  // Load into existing Preferences window
  let enumerator = Services.wm.getEnumerator(SAP_prefWindowType);
  while (enumerator.hasMoreElements())
    loadIntoWindow(enumerator.getNext());

  // Load into any new windows
  Services.wm.addListener(windowListener);
}

function shutdown(aData, aReason) {
  // When the application is shutting down
  // we normally don't have to clean up any UI changes
  if (aReason == APP_SHUTDOWN || !isSM28(aData))
    return;

  // Reset prefs
  Services.prefs.clearUserPref("services.sync.registerEngines");
  Services.prefs.clearUserPref("services.sync.prefs.sync.addons.ignoreUserEnabledChanges");

  // Stop watching for new windows
  Services.wm.removeListener(windowListener);

  // Unload from existing Preferences window
  let enumerator = Services.wm.getEnumerator(SAP_prefWindowType);
  while (enumerator.hasMoreElements())
    unloadFromWindow(enumerator.getNext());
}

function install(aData, aReason) { }

function uninstall(aData, aReason) { }

